package hu.nticket.test.infinispan;

import java.util.Date;
import java.util.concurrent.atomic.AtomicInteger;

import javax.cache.Cache;
import javax.cache.CacheManager;
import javax.cache.Caching;

public class Main {

    public static class TestClassLoader extends ClassLoader {
        public TestClassLoader(ClassLoader parent) {
            super(parent);
        }
    }

    public static void main(String[] args) {
        // Distribution cache cannot run with more than 2 instances of CacheManagers. Therefore it was commented out
        // from the xml as well.
        // testDistributedCache();
        testReplicationCache();
    }

    public static void testReplicationCache() {
        final int THREAD_NUM = 10;
        final int ITERATION_NUM = 10000;

        ClassLoader tccl = Thread.currentThread().getContextClassLoader();
        CacheManager cacheManager1 = Caching
                .getCacheManager(new TestClassLoader(tccl), "infinispan-jcache-cluster.xml");
        CacheManager cacheManager2 = Caching
                .getCacheManager(new TestClassLoader(tccl), "infinispan-jcache-cluster.xml");
        CacheManager cacheManager3 = Caching
                .getCacheManager(new TestClassLoader(tccl), "infinispan-jcache-cluster.xml");
        CacheManager cacheManager4 = Caching
                .getCacheManager(new TestClassLoader(tccl), "infinispan-jcache-cluster.xml");
        CacheManager cacheManager5 = Caching
                .getCacheManager(new TestClassLoader(tccl), "infinispan-jcache-cluster.x ml");

        final Cache<String, String> cache1 = cacheManager1.getCache("replicatedCache");
        final Cache<String, String> cache2 = cacheManager2.getCache("replicatedCache");
        cacheManager3.getCache("replicatedCache");
        cacheManager4.getCache("replicatedCache");
        cacheManager5.getCache("replicatedCache");

        System.out.println("Caches started");

        long startTime = new Date().getTime();

        final AtomicInteger runningThreads = new AtomicInteger(0);
        final AtomicInteger cacheMiss = new AtomicInteger(0);
        for (int i = 0; i < THREAD_NUM; i++) {
            runningThreads.incrementAndGet();
            final int threadIndex = i;
            Thread thread = new Thread(new Runnable() {

                public void run() {
                    for (int j = 0; j < ITERATION_NUM; j++) {
                        cache1.put(threadIndex + "-hello-" + j, "world" + j);
                        String value = cache2.get(threadIndex + "-hello-" + j);
                        if (value == null || !value.equals("world" + j)) {
                            cacheMiss.incrementAndGet();
                        }
                    }
                    System.out.flush();
                    runningThreads.decrementAndGet();
                }
            });
            thread.start();
        }

        while (runningThreads.get() > 0) {
            try {
                Thread.sleep(10);
            } catch (InterruptedException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }
        System.out.println("Tests were running " + (new Date().getTime() - startTime) + " ms. long");
        System.out.println("Cache miss: " + cacheMiss.get());
        startTime = new Date().getTime();

        for (int i = 0; i < THREAD_NUM; i++) {
            for (int j = 0; j < ITERATION_NUM; j++) {
                cache2.get(i + "-hello-" + j);
            }
        }
        System.out.println("Gets were running " + (new Date().getTime() - startTime) + " ms. long");
        startTime = new Date().getTime();
        cacheMiss.set(0);
        for (int i = 0; i < THREAD_NUM; i++) {
            runningThreads.incrementAndGet();
            final int threadIndex = i;
            Thread thread = new Thread(new Runnable() {

                public void run() {
                    for (int j = 0; j < ITERATION_NUM; j++) {
                        cache1.remove(threadIndex + "-hello-" + j, "world" + j);
                        String value = cache2.get(threadIndex + "-hello-" + j);
                        if (value != null) {
                            cacheMiss.incrementAndGet();
                        }
                    }
                    runningThreads.decrementAndGet();
                }
            });
            thread.start();
        }

        while (runningThreads.get() > 0) {
            try {
                Thread.sleep(10);
            } catch (InterruptedException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }

        System.out.println("Deletes were running " + (new Date().getTime() - startTime) + " ms. long");
        System.out.println("Cache miss: " + cacheMiss.get());
        startTime = new Date().getTime();

        Caching.close();

    }

    public static void testDistributedCache() {
        final int THREAD_NUM = 100;
        final int ITERATION_NUM = 10000;

        ClassLoader tccl = Thread.currentThread().getContextClassLoader();
        CacheManager cacheManager1 = Caching
                .getCacheManager(new TestClassLoader(tccl), "infinispan-jcache-cluster.xml");
        CacheManager cacheManager2 = Caching
                .getCacheManager(new TestClassLoader(tccl), "infinispan-jcache-cluster.xml");
        CacheManager cacheManager3 = Caching
                .getCacheManager(new TestClassLoader(tccl), "infinispan-jcache-cluster.xml");
        // CacheManager cacheManager4 = Caching
        // .getCacheManager(new TestClassLoader(tccl), "infinispan-jcache-cluster.xml");
        // CacheManager cacheManager5 = Caching
        // .getCacheManager(new TestClassLoader(tccl), "infinispan-jcache-cluster.xml");

        final Cache<String, String> cache1 = cacheManager1.getCache("distributionCache");
        final Cache<String, String> cache2 = cacheManager2.getCache("distributionCache");
        cacheManager3.getCache("distributionCache");
        // cacheManager4.getCache("distributionCache");
        // cacheManager5.getCache("distributionCache");

        long startTime = new Date().getTime();

        final AtomicInteger runningThreads = new AtomicInteger(0);
        for (int i = 0; i < THREAD_NUM; i++) {
            runningThreads.incrementAndGet();
            final int threadIndex = i;
            Thread thread = new Thread(new Runnable() {

                public void run() {
                    for (int j = 0; j < ITERATION_NUM; j++) {
                        cache1.put(threadIndex + "-hello-" + j, "world" + j);
                        // cache2.get(threadIndex + "-hello-" + j);
                        // No cache2 check here to see how much faster an insert is.
                    }
                    runningThreads.decrementAndGet();
                }
            });
            thread.start();
        }

        while (runningThreads.get() > 0) {
            try {
                Thread.sleep(10);
            } catch (InterruptedException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }
        System.out.println("Tests were running " + (new Date().getTime() - startTime) + " ms. long");
        final AtomicInteger cacheMissNum = new AtomicInteger(0);
        startTime = new Date().getTime();

        for (int i = 0; i < THREAD_NUM; i++) {
            runningThreads.incrementAndGet();
            final int threadIndex = i;
            new Thread(new Runnable() {

                public void run() {
                    for (int j = 0; j < ITERATION_NUM; j++) {
                        String value = cache2.get(threadIndex + "-hello-" + j);
                        if (value == null || !value.equals("world" + j)) {
                            cacheMissNum.incrementAndGet();
                        }
                    }
                    runningThreads.decrementAndGet();
                }
            }).start();

        }

        while (runningThreads.get() > 0) {
            try {
                Thread.sleep(10);
            } catch (InterruptedException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }
        System.out.println("Cache miss: " + cacheMissNum.get());
        System.out.println("Gets were running " + (new Date().getTime() - startTime) + " ms. long");
        startTime = new Date().getTime();

        cacheMissNum.set(0);
        for (int i = 0; i < THREAD_NUM; i++) {
            runningThreads.incrementAndGet();
            final int threadIndex = i;
            new Thread(new Runnable() {

                public void run() {
                    for (int j = 0; j < ITERATION_NUM; j++) {
                        String value = cache2.get(threadIndex + "-hello-" + j);
                        if (value == null || !value.equals("world" + j)) {
                            cacheMissNum.incrementAndGet();
                        }
                    }
                    runningThreads.decrementAndGet();
                }
            }).start();

        }

        while (runningThreads.get() > 0) {
            try {
                Thread.sleep(10);
            } catch (InterruptedException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }
        System.out.println("Cache miss: " + cacheMissNum.get());
        System.out.println("Gets were running " + (new Date().getTime() - startTime) + " ms. long");
        startTime = new Date().getTime();

        for (int i = 0; i < THREAD_NUM; i++) {
            runningThreads.incrementAndGet();
            final int threadIndex = i;
            Thread thread = new Thread(new Runnable() {

                public void run() {
                    for (int j = 0; j < ITERATION_NUM; j++) {
                        cache1.remove(threadIndex + "-hello-" + j, "world" + j);
                        String value = cache2.get(threadIndex + "-hello-" + j);
                        if (value != null) {
                            System.out.println("Cache mismatch");
                        }
                    }
                    System.out.flush();
                    runningThreads.decrementAndGet();
                }
            });
            thread.start();
        }

        while (runningThreads.get() > 0) {
            try {
                Thread.sleep(10);
            } catch (InterruptedException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }

        System.out.println("Deletes were running " + (new Date().getTime() - startTime) + " ms. long");
        startTime = new Date().getTime();

        Caching.close();

    }

    public static void testInvalidationCache() {
        final int THREAD_NUM = 1;
        final int ITERATION_NUM = 1;

        ClassLoader tccl = Thread.currentThread().getContextClassLoader();
        CacheManager cacheManager1 = Caching
                .getCacheManager(new TestClassLoader(tccl), "infinispan-jcache-cluster.xml");
        CacheManager cacheManager2 = Caching
                .getCacheManager(new TestClassLoader(tccl), "infinispan-jcache-cluster.xml");

        final Cache<String, String> cache1 = cacheManager1.getCache("invalidationCache");
        final Cache<String, String> cache2 = cacheManager2.getCache("invalidationCache");

        long startTime = new Date().getTime();

        final AtomicInteger runningThreads = new AtomicInteger(0);
        for (int i = 0; i < THREAD_NUM; i++) {
            runningThreads.incrementAndGet();
            final int threadIndex = i;
            Thread thread = new Thread(new Runnable() {

                public void run() {
                    for (int j = 0; j < ITERATION_NUM; j++) {
                        String key = threadIndex + "-hello-" + j;
                        String insertingValue = "world" + j;
                        cache1.put(key, insertingValue);
                        boolean contains = cache2.containsKey(key);
                        String returningValue = cache2.get(key);
                        if (returningValue != null) {
                            System.out.println("Cache mismatch");
                        }
                    }
                    System.out.flush();
                    runningThreads.decrementAndGet();
                }
            });
            thread.start();
        }

        while (runningThreads.get() > 0) {
            try {
                Thread.sleep(10);
            } catch (InterruptedException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }
        System.out.println("Cache 1 fill was running " + (new Date().getTime() - startTime) + " ms. long");
        startTime = new Date().getTime();

        // ------------------------------------------------------------------------------------------------------

        for (int i = 0; i < THREAD_NUM; i++) {
            runningThreads.incrementAndGet();
            final int threadIndex = i;
            Thread thread = new Thread(new Runnable() {

                public void run() {
                    for (int j = 0; j < ITERATION_NUM; j++) {
                        String key = threadIndex + "-hello-" + j;
                        String insertingValue = "world" + (j);
                        String beforeValue = cache1.get(key);
                        cache2.put(key, insertingValue);
                        String returningValue = cache1.get(key);
                        if (returningValue == null || returningValue.equals(insertingValue)) {
                            System.out.println("Cache mismatch");
                        }
                    }
                    System.out.flush();
                    runningThreads.decrementAndGet();
                }
            });
            thread.start();
        }

        while (runningThreads.get() > 0) {
            try {
                Thread.sleep(10);
            } catch (InterruptedException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }
        System.out.println("Cache 2 fill was running " + (new Date().getTime() - startTime) + " ms. long");
        startTime = new Date().getTime();

        for (int i = 0; i < THREAD_NUM; i++) {
            for (int j = 0; j < ITERATION_NUM; j++) {
                cache2.get(i + "-hello-" + j);
            }
        }
        System.out.println("Gets were running " + (new Date().getTime() - startTime) + " ms. long");
        startTime = new Date().getTime();

        for (int i = 0; i < THREAD_NUM; i++) {
            runningThreads.incrementAndGet();
            final int threadIndex = i;
            Thread thread = new Thread(new Runnable() {

                public void run() {
                    for (int j = 0; j < ITERATION_NUM; j++) {
                        cache1.remove(threadIndex + "-hello-" + j, "world" + j);
                        String value = cache2.get(threadIndex + "-hello-" + j);
                        if (value != null) {
                            System.out.println("Cache mismatch");
                        }
                    }
                    System.out.flush();
                    runningThreads.decrementAndGet();
                }
            });
            thread.start();
        }

        while (runningThreads.get() > 0) {
            try {
                Thread.sleep(10);
            } catch (InterruptedException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }

        System.out.println("Deletes were running " + (new Date().getTime() - startTime) + " ms. long");
        startTime = new Date().getTime();

        Caching.close();
    }
}
